/*
 * Copyright (c) 2003-2009 jMonkeyEngine
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of 'jMonkeyEngine' nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software 
 *   without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.titusgames.jglest.playground;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.jme.app.SimpleGame;
import com.jme.app.SimplePassGame;
import com.jme.input.MouseInput;
import com.jme.light.PointLight;
import com.jme.math.Plane;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.renderer.pass.RenderPass;
import com.jme.scene.Node;
import com.jme.scene.SharedNode;
import com.jme.scene.Spatial;
import com.jme.scene.shape.Quad;
import com.jme.util.CloneImportExport;
import com.jme.util.resource.ResourceLocator;
import com.jme.util.resource.ResourceLocatorTool;
import com.jmex.effects.water.WaterRenderPass;
import com.jmex.model.converters.FormatConverter;

import de.titusgames.jglest.model.G3dModelLoader;
import de.titusgames.jglest.model.GTerrain;
import de.titusgames.jglest.terrain.TerrainLoader;
import de.titusgames.jglest.tileset.Tileset;
import de.titusgames.jglest.tileset.TilesetModels;
import de.titusgames.jglest.tileset.TilesetObject;
import de.titusgames.jglest.xml.parser.GbmXmlReader;
import de.titusgames.jglest.xml.parser.TilesetXmlReader;

/**
 * Started Date: Jul 22, 2004<br>
 * <br>
 * 
 * Demonstrates loading formats.
 * 
 * @author Jack Lindamood
 */
public class HelloModelLoading extends SimplePassGame
{
   private static final Logger logger = Logger.getLogger(HelloModelLoading.class.getName());

   public static void main(String[] args)
   {
      HelloModelLoading app = new HelloModelLoading();
      app.setConfigShowMode(ConfigShowMode.AlwaysShow);
      // Turn the logger off so we can see the XML later on
      app.start();
   }

   //ShadowedRenderPass sPass;

   protected void simpleInitGame()
   {
      // Point to a URL of my model
      // File file = new File("resources/acolyte_walking_gold.xml");
      File file = new File("resources/battlemage_statue.xml");
      File mapfile = new File("resources/Wadi Nefud2.xml");
      URL model = null;

      
      ResourceLocator myLocator = new ResourceLocator()
      {
         public URL locateResource(String resourceName)
         {
            try
            {
               return new URL("file://"+resourceName);
            }
            catch (MalformedURLException e)
            {
               e.printStackTrace();
               return null;
            }
         }
      };

      ResourceLocatorTool.addResourceLocator(
            ResourceLocatorTool.TYPE_TEXTURE, myLocator);
 
      
      // Initialize camera
      cam.setFrustumPerspective(45.0f, (float) display.getWidth()
            / (float) display.getHeight(), 1f, 1000f);
      cam.setLocation(new Vector3f(0, 160, 10));
      cam.lookAt(new Vector3f(0, 0, 0), Vector3f.UNIT_Y);
      cam.update();

      PointLight dr = new PointLight();
      dr.setEnabled(true);
      dr.setDiffuse(new ColorRGBA(1.0f, 1.0f, 1.0f, 1.0f));
      dr.setAmbient(new ColorRGBA(0.5f, 0.5f, 0.5f, 1.0f));
      dr.setLocation(new Vector3f(500.0f, 500.0f, 0));
      dr.setShadowCaster(true);

      lightState.detachAll();
      lightState.attach(dr);
      lightState.setTwoSidedLighting(true);

     

      try
      {
         // /////////////////////
         // Terrain
         // /////////////////////
         Tileset tileset;
         TilesetXmlReader tilesetXmlReader;
         GbmXmlReader gbmXmlReader = new GbmXmlReader();
         TerrainLoader terrainLoader;
         try
         {
            GTerrain terrain = gbmXmlReader.readGbmXml(mapfile);

            // rootNode.attachChild(splattingPassNode);

            tilesetXmlReader = new TilesetXmlReader();
            tileset = tilesetXmlReader.readXml(new File(
                  "resources/evergreen/evergreen.xml"));
            terrainLoader = new TerrainLoader("resources/evergreen", display);
            Spatial ground = terrainLoader.createTerrain(
                  terrain, tileset.getSurfaces());

            // sPass = new ShadowedRenderPass();
            // sPass.add(rootNode);
            // sPass.addOccluder(ground);
            // sPass.setRenderShadows(true);
            // sPass.setLightingMethod(LightingMethod.Additive);
            //            pManager.add(sPass);
            
            
            rootNode.attachChild(ground);

            

            WaterRenderPass waterEffectRenderPass=createWater();
            
            //pManager.add(waterEffectRenderPass);

            Quad waterQuad = new Quad("waterQuad", 10, 10);
            FloatBuffer normBuf = waterQuad.getNormalBuffer();
            normBuf.clear();
            normBuf.put(0).put(-1).put(0);
            normBuf.put(0).put(-1).put(0);
            normBuf.put(0).put(-1).put(0);
            normBuf.put(0).put(-1).put(0);
            
            
            FloatBuffer vertBuf = waterQuad.getVertexBuffer();
            vertBuf.clear();

            float height=terrain.getWaterLevel()*new Float(terrain.getAltFactor())*0.1f;
            waterEffectRenderPass.setWaterHeight(height);
            vertBuf.put(terrain.getWidth()).put(height).put(-terrain.getHeight());
            vertBuf.put(terrain.getWidth()).put(height).put(terrain.getHeight());
            vertBuf.put(-terrain.getWidth()).put(height).put(terrain.getHeight());
            vertBuf.put(-terrain.getWidth()).put(height).put(-terrain.getHeight());
            
            //waterQuad.rotatePoints(new Quaternion(0.0f,0.0f,90.0f,0.0f));
            
            

            
            //waterEffectRenderPass.setWaterEffectOnSpatial(waterQuad);
            
            fillTerrainWithModels(waterEffectRenderPass, rootNode, tileset, terrain);

            //waterEffectRenderPass.addReflectedScene(ground);
            
            rootNode.attachChild(waterQuad);

            RenderPass rootPass = new RenderPass();
            rootPass.add(rootNode);
            pManager.add(rootPass);

            
         }
         catch (Exception ex)
         {
            ex.printStackTrace();
         }
         // /////////////////////
         // Model loading
         // /////////////////////
         model = file.toURI().toURL();

         // URL
         // model=HelloModelLoading.class.getClassLoader().getResource("resources/acolyte_standing.xml");

         // Create something to convert .obj format to .jme
         FormatConverter converter = new GxmlToJme();
         converter.setProperty(
               "texturedir", file.getParentFile().toURI().toURL());
         // // Point the converter to where it will find the .mtl file from
         // converter.setProperty("mtllib", model);

         // This byte array will hold my .jme file
         ByteArrayOutputStream BO = new ByteArrayOutputStream();
         try
         {
            G3dModelLoader modelLoader = new G3dModelLoader();
            URL baseURL = new File("resources/").toURI().toURL();
            String path = "battlemage_statue.xml";
            Node maggie = modelLoader.loadModel(baseURL, path);

            // // Use the format converter to convert .obj to .jme
            // converter.convert(model.openStream(), BO);
            // Node maggie = (Node) BinaryImporter.getInstance().load(
            // new ByteArrayInputStream(BO.toByteArray()));
            // // shrink this baby down some
            // maggie.setLocalScale(10.0f);
            // maggie.setModelBound(new BoundingSphere());
            // maggie.updateModelBound();

            // Put her on the scene graph
            rootNode.attachChild(maggie);
            MouseInput.get().setCursorVisible(true);
         }
         catch (Exception e)
         { // Just in case anything happens
            logger.logp(
                  Level.SEVERE, this.getClass().toString(), "simpleInitGame()",
                  "Exception", e);
            System.exit(0);
         }
      }
      catch (MalformedURLException e1)
      {
         // TODO Auto-generated catch block
         e1.printStackTrace();
      }
   }

   private void fillTerrainWithModels(
         WaterRenderPass waterEffectRenderPass,Node rootNode, Tileset tileset, GTerrain terrain) throws IOException,
         URISyntaxException
   {
      TilesetModels tilesetModels = readModelsFromTileset(tileset);

      for (int y = 0; y < terrain.getHeight(); y++)
      {
         for (int x = 0; x < terrain.getWidth(); x++)
         {
            int i = terrain.getObjects()[y * terrain.getWidth() + x];
            if (i != 0)
            {
               Node node = tilesetModels.getNodeForNumber(i - 1);
               if (node != null)
               {
                  //Node newNode = getCloneNode(node);
                  SharedNode newNode=new SharedNode("clone index"+y * terrain.getWidth() + x,node);
                  newNode.setLocalTranslation(new Vector3f(x
                        - terrain.getWidth() / 2,
                        terrain.getAltFactor()
                              * terrain.getTerrainHeight()[y
                                    * terrain.getWidth() + x], y
                              - terrain.getHeight() / 2));
                  newNode.lock();
                  waterEffectRenderPass.addReflectedScene(newNode);
                  rootNode.attachChild(newNode);
               }
            }
         }
      }
   }

   G3dModelLoader modelLoader = new G3dModelLoader();

   private TilesetModels readModelsFromTileset(Tileset tileset)
         throws IOException, URISyntaxException
   {
      TilesetModels tilesetModels = new TilesetModels();
      int i = 0;
      for (TilesetObject tilesetObject : tileset.getObjects())
      {
         i++;
         System.out.println("i vorher=" + i);
         ArrayList<Node> nodeList = new ArrayList<Node>();
         tilesetModels.addNodeList(nodeList);
         for (String relPath : tilesetObject.getModelPaths())
         {
            if (relPath.equals("models/none.xml"))
            {
               nodeList.add(null);
            }
            else
            {
               //System.out.println("---------3");
               Node node = modelLoader.loadModel(tileset.getBaseUrl(), relPath);
               nodeList.add(node);
               //System.out.println("---------4");
            }
         }
      }
      return tilesetModels;
   }
   CloneImportExport ie = new CloneImportExport();
   private Node getCloneNode(Node node)
   {
      ie.saveClone(node);
      Node nodeneu = (Node) ie.loadClone();
      return nodeneu;
   }

   private WaterRenderPass createWater()
   {
      WaterRenderPass waterEffectRenderPass = new WaterRenderPass(cam, 1,
            false, false);

      Plane waterplane = new Plane(new Vector3f(0.0f, 1.0f, 0.0f), 0.0f);
      waterEffectRenderPass.setWaterPlane(waterplane);
      waterEffectRenderPass.setClipBias(-1.0f);
      waterEffectRenderPass.setReflectionThrottle(0.0f);
      waterEffectRenderPass.setRefractionThrottle(0.0f);
      waterEffectRenderPass.setWaterMaxAmplitude(5.0f);

      return waterEffectRenderPass;
      // waterEffectRenderPass.setReflectedScene(skybox);
      // waterEffectRenderPass.addReflectedScene(splatTerrain);
      // waterEffectRenderPass.addReflectedScene(arena);
      // waterEffectRenderPass.setSkybox(skybox);
   }

   @Override
   protected void simpleUpdate()
   {
      // if (waterEffectRenderPass != null && waterEffectRenderPass.isEnabled())
      // {
      // Vector3f transVec = new Vector3f(cam.getLocation().x,
      // waterEffectRenderPass.getWaterHeight(), cam.getLocation().z);
      // setTextureCoords(0, transVec.x, -transVec.z, textureScale);
      // setVertexCoords(transVec.x, transVec.y, transVec.z);
      // }
      super.simpleUpdate();
   }
}
