package de.titusgames.jglest.terrain;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.List;

import com.jme.image.Texture;
import com.jme.math.Vector3f;
import com.jme.scene.PassNode;
import com.jme.scene.PassNodeState;
import com.jme.scene.Spatial;
import com.jme.scene.state.BlendState;
import com.jme.scene.state.TextureState;
import com.jme.system.DisplaySystem;
import com.jme.util.TextureManager;
import com.jmex.terrain.TerrainPage;

import de.titusgames.jglest.model.GTerrain;
import de.titusgames.jglest.tileset.TilesetSurface;

public class TerrainLoader
{
   DisplaySystem display;
   String basePath;

   public TerrainLoader(String basePath,DisplaySystem display)
   {
      super();
      this.display = display;
      this.basePath=basePath;
   }

   public Spatial createTerrain(GTerrain terrain, List<TilesetSurface> surfaces)
         throws IOException
   {
      
      // /////////////////////
      // Terrain
      // /////////////////////

      System.out.println("terrain.getAltFactor()=" + terrain.getAltFactor());

      
      float[] smoothHeight=terrain.smoothHeight(terrain.getTerrainHeight(), terrain.getWidth(), terrain.getHeight());
      smoothHeight=terrain.smoothHeight(smoothHeight, terrain.getWidth(), terrain.getHeight());
      
      terrain.setTerrainHeight(smoothHeight);
      
      // creating the base Terrain
      TerrainPage tb = new TerrainPage("Terrain", terrain.getWidth(), terrain.getWidth()+ 1, new Vector3f(1,
            terrain.getAltFactor(), 1), smoothHeight);
      //tb.getLocalTranslation().set(0, 10, 0);
      tb.setDetailTexture(1, 1);

      // Add a base texture ( maybe its needed one day )
      // URL grass = HelloTerrain.class.getClassLoader().getResource(
      // "jmetest/data/texture/grassb.png");
      // TextureState ts = display.getRenderer().createTextureState();
      //
      // Texture texture = TextureManager.loadTexture(
      // grass, Texture.MinificationFilter.BilinearNearestMipMap,
      // Texture.MagnificationFilter.Bilinear);
      // ts.setTexture(texture);

      // just for debug .................
      // JFrame frame =new JFrame();
      // JPanel panel=new JPanel();
      // panel.setBackground(Color.BLACK);
      // JLabel button1=new JLabel(new ImageIcon(alphas[0]));
      // JLabel button2=new JLabel(new ImageIcon(alphas[1]));
      // button1.setPreferredSize(new Dimension(256, 256));
      // button2.setPreferredSize(new Dimension(256, 256));
      // button1.setBackground(Color.BLACK);
      // panel.add(button1);
      // panel.add(button2);
      // frame.getContentPane().add(panel);
      // frame.setSize(new Dimension(800,600));
      // frame.setVisible(true);

      // alpha used for blending the passnodestates together
      BlendState as = display.getRenderer().createBlendState();
      as.setBlendEnabled(true);
      as.setSourceFunction(BlendState.SourceFunction.SourceAlpha);
      as.setDestinationFunction(BlendState.DestinationFunction.OneMinusSourceAlpha);
      as.setTestEnabled(true);
      as.setTestFunction(BlendState.TestFunction.GreaterThan);
      as.setEnabled(true);

      BufferedImage[] alphas = new AlphaCalculator().createAlphaTexture(
            terrain.getSurfaces(), terrain.getHeight(), terrain.getWidth(),
            surfaces.size());

      PassNode splattingPassNode = new PassNode("SplatPassNode");
      splattingPassNode.attachChild(tb);

      // adding all the TextureStates for each surface
      for (int i = 0; i < surfaces.size(); i++)
      {
         TextureState ts1 = createSplatTextureState(
               createTextureImage(surfaces.get(i)), alphas[i]);
         PassNodeState passNodeState = new PassNodeState();
         passNodeState.setPassState(ts1);
         passNodeState.setPassState(as);
         splattingPassNode.addPass(passNodeState);
      }

      // lock some things to increase the performance
      splattingPassNode.lockBounds();
      splattingPassNode.lockTransforms();
      splattingPassNode.lockShadows();

      return splattingPassNode;

   }

   BufferedImage createTextureImage(TilesetSurface surface) throws IOException
   {
      TextureMerger tm = new TextureMerger();
      BufferedImage bufferedImage = tm.createTexture(basePath,surface.getTextures());
      return bufferedImage;

   }

//   BufferedImage createTextureImage2() throws IOException
//   {
//      ArrayList<PathChance> pathChanceList = new ArrayList<PathChance>();
//      pathChanceList.add(new PathChance("resources/autumn/surface3a.bmp", 0.4f));
//      pathChanceList.add(new PathChance("resources/autumn/surface3b.bmp", 0.6f));
//      // pathChanceList.add(new PathChance("resources/autumn/surface2c.bmp",
//      // 0.3f));
//      // pathChanceList.add(new PathChance("resources/autumn/surface2d.bmp",
//      // 0.21f));
//      // pathChanceList.add(new PathChance("resources/autumn/surface2e.bmp",
//      // 0.10f));
//      // pathChanceList.add(new PathChance("resources/autumn/surface2f.bmp",
//      // 0.02f));
//
//      TextureMerger tm = new TextureMerger();
//      BufferedImage backImg = tm.createTexture(pathChanceList);
//      return backImg;
//
//   }

   private TextureState createSplatTextureState(
         java.awt.Image image, java.awt.Image alpha)
   {
      TextureState ts = display.getRenderer().createTextureState();

      Texture t0 = TextureManager.loadTexture(
            image, Texture.MinificationFilter.BilinearNearestMipMap,
            Texture.MagnificationFilter.Bilinear, false);

      t0.setWrap(Texture.WrapMode.Repeat);
      t0.setApply(Texture.ApplyMode.Modulate);
      t0.setScale(new Vector3f(8.0f, 8.0f, 1.0f));

      ts.setTexture(t0, 0);

      if (alpha != null)
      {
         addAlphaSplat(ts, alpha);
      }

      return ts;
   }

   private void addAlphaSplat(TextureState ts, java.awt.Image alpha)
   {
      Texture t1 = TextureManager.loadTexture(
            alpha, Texture.MinificationFilter.BilinearNearestMipMap,
            Texture.MagnificationFilter.Bilinear, false);
      t1.setWrap(Texture.WrapMode.Repeat);
      t1.setApply(Texture.ApplyMode.Combine);
      t1.setCombineFuncRGB(Texture.CombinerFunctionRGB.Replace);
      t1.setCombineSrc0RGB(Texture.CombinerSource.Previous);
      t1.setCombineOp0RGB(Texture.CombinerOperandRGB.SourceColor);
      t1.setCombineFuncAlpha(Texture.CombinerFunctionAlpha.Replace);
      ts.setTexture(t1, ts.getNumberOfSetTextures());
   }

}
