package de.titusgames.jglest.xml.parser;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.jme.math.Vector2f;
import com.jme.math.Vector3f;

import de.titusgames.jglest.model.G3dModel;
import de.titusgames.jglest.model.GMesh;

public class G3dXmlSaxHandler extends XmlSaxHandlerBase
{

   private G3dModel            model;
   private GMesh               mesh;
   private Vector3f[]          array3f;
   private Vector3f            vector3f;
   private Vector2f            vector2f;
   private Vector2f[]          array2f;
   private int[]               intArray;
   private int                 i                   = 0;

   private static final String NODE_G3D            = "G3D";
   private static final String NODE_Mesh           = "Mesh";
   private static final String NODE_Diffuse        = "Diffuse";
   private static final String NODE_Color          = "Color";
   private static final String NODE_Specular       = "Specular";
   private static final String NODE_Texture        = "Texture";
   private static final String NODE_Vertices       = "Vertices";
   private static final String NODE_Vertex         = "Vertex";
   private static final String NODE_Normals        = "Normals";
   private static final String NODE_Normal         = "Normal";
   private static final String NODE_TexCoords      = "TexCoords";
   private static final String NODE_ST             = "ST";
   private static final String NODE_Indices        = "Indices";
   private static final String NODE_Ix             = "Ix";

   private static final String ATTR_version        = "version";
   private static final String ATTR_name           = "name";
   private static final String ATTR_frameCount     = "frameCount";
   private static final String ATTR_vertexCount    = "vertexCount";
   private static final String ATTR_indexCount     = "indexCount";
   private static final String ATTR_specularPower  = "specularPower";
   private static final String ATTR_opacity        = "opacity";
   private static final String ATTR_twoSided       = "twoSided";
   private static final String ATTR_customColor    = "customColor";
   private static final String ATTR_diffuseTexture = "diffuseTexture";
   private static final String ATTR_r              = "r";
   private static final String ATTR_g              = "g";
   private static final String ATTR_b              = "b";
   private static final String ATTR_x              = "x";
   private static final String ATTR_y              = "y";
   private static final String ATTR_z              = "z";
   private static final String ATTR_i              = "i";
   private static final String ATTR_s              = "s";
   private static final String ATTR_t              = "t";
   //private static final String ATTR_frame          = "frame";

   // public static final String NODE_ ="";
   // public static final String NODE_ ="";

   // <G3D version="4">
   // <Mesh name="worker_stand_None" frameCount="20" vertexCount="116"
   // indexCount="282" specularPower="9.999999" opacity="1.000000"
   // twoSided="true" customColor="true" diffuseTexture="true">
   // <Diffuse>
   // <Color r="0.588235" g="0.588235" b="0.588235"/>
   // </Diffuse>
   // <Specular>
   // <Color r="0.900000" g="0.900000" b="0.900000"/>
   // </Specular>

   public G3dXmlSaxHandler()
   {
      super();
   }

   public void startElement(
         String uri, String localName, String qName, Attributes attributes)
         throws SAXException
   {
      // super.startElement(uri, localName, qName, attributes);
      // String nodeName = localName;
      // if ((nodeName == null) || nodeName.equals(""))
      // {
      // nodeName = qName;
      // }
      if (NODE_G3D.equals(qName))
      {
         model = new G3dModel(uri);
         model.setVersion(attributes.getValue(ATTR_version));
      }
      else if (NODE_Mesh.equals(qName))
      {
         mesh = new GMesh();
         model.add(mesh);
         mesh.setName(attributes.getValue(ATTR_name));
         mesh.setFrameCount(getIntegerValue(attributes, ATTR_frameCount));
         mesh.setVertexCount(getIntegerValue(attributes, ATTR_vertexCount));
         mesh.setIndexCount(getIntegerValue(attributes, ATTR_indexCount));
         mesh.setSpecularPower(getFloatValue(attributes, ATTR_specularPower));
         mesh.setOpacity(getFloatValue(attributes, ATTR_opacity));
         mesh.setCustomColor(getBooleanValue(attributes, ATTR_customColor));
         mesh.setTwoSided(getBooleanValue(attributes, ATTR_twoSided));
         mesh.setDiffuseTexture(getBooleanValue(attributes, ATTR_diffuseTexture));
         
         // mesh.set(attributes.getValue(ATTR_));
         // mesh.set(attributes.getValue(ATTR_));
         // mesh.set(attributes.getValue(ATTR_));
      }
      else if (NODE_Diffuse.equals(qName))
      {
         vector3f = new Vector3f();
         mesh.setDiffuseColor(vector3f);
      }
      else if (NODE_Color.equals(qName))
      {
         vector3f.setX(getFloatValue(attributes, ATTR_r));
         vector3f.setY(getFloatValue(attributes, ATTR_g));
         vector3f.setZ(getFloatValue(attributes, ATTR_b));
      }
      else if (NODE_Specular.equals(qName))
      {
         vector3f = new Vector3f();
         mesh.setSpecularColor(vector3f);
      }
      else if (NODE_Texture.equals(qName))
      {
         mesh.addTexturePath(attributes.getValue(ATTR_name));
      }
      else if (NODE_Vertices.equals(qName))
      {
         array3f = new Vector3f[mesh.getVertexCount()];
         mesh.addVertices(array3f);
         i = 0;
      }
      else if (NODE_Vertex.equals(qName))
      {
         vector3f = new Vector3f();
         vector3f.setX(getFloatValue(attributes, ATTR_x));
         vector3f.setY(getFloatValue(attributes, ATTR_y));
         vector3f.setZ(getFloatValue(attributes, ATTR_z));
         array3f[i] = vector3f;
         i++;
      }
      else if (NODE_Normals.equals(qName))
      {
         array3f = new Vector3f[mesh.getVertexCount()];
         mesh.addNormals(array3f);
         i = 0;
      }
      else if (NODE_Normal.equals(qName))
      {
         vector3f = new Vector3f();
         vector3f.setX(getFloatValue(attributes, ATTR_x));
         vector3f.setY(getFloatValue(attributes, ATTR_y));
         vector3f.setZ(getFloatValue(attributes, ATTR_z));
         array3f[i] = vector3f;
         i++;
      }
      else if (NODE_TexCoords.equals(qName))
      {
         array2f = new Vector2f[mesh.getVertexCount()];
         mesh.setTexCoords(array2f);
         i = 0;
      }
      else if (NODE_ST.equals(qName))
      {
         vector2f = new Vector2f();
         vector2f.setX(getFloatValue(attributes, ATTR_s));
         vector2f.setY(getFloatValue(attributes, ATTR_t));
         array2f[i] = (vector2f);
         i++;
      }
      else if (NODE_Indices.equals(qName))
      {
         intArray = new int[mesh.getIndexCount()];
         mesh.setIndices(intArray);
         i = 0;
      }
      else if (NODE_Ix.equals(qName))
      {
         intArray[i] = getIntegerValue(attributes, ATTR_i);
         i++;
      }
      else
      {
         throw new SAXException("Cannot handle node named '" + qName + "'");
      }

   }

   public void endElement(String uri, String localName, String qName)
         throws SAXException
   {

   }

   public InputSource resolveEntity(String publicId, String systemId)
         throws SAXException
   {
      InputSource inputSource = new InputSource();
      systemId = "g3d.dtd";
      inputSource.setSystemId(systemId);
      // inputSource.setPublicId("messungsdatei");
      return inputSource;
   }

   public G3dModel getModel()
   {
      return model;
   }

}
