package de.titusgames.jglest.xml.parser;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.titusgames.jglest.model.GPlayer;
import de.titusgames.jglest.model.GTerrain;

public class GbmXmlSaxHandler extends XmlSaxHandlerBase
{
   GTerrain                   terrain;
   GPlayer                    currentGplayer;
   int                        i               = 0;

   // <gbm version="1" maxPlayers="4" width="64" height="64" altFactor="3"
   // waterLevel="4" >
   // <title name=""/>
   // <author name=""/>
   // <description name=""/>
   // <player num="0" posx="53" posy="42" />
   // <player num="1" posx="53" posy="53" />
   // <player num="2" posx="10" posy="10" />
   // <player num="3" posx="10" posy="21" />
   // <!-- height ****************** -->
   // <h v="8.000000" />
   public static final String NODE_gbm        = "gbm";
   public static final String NODE_player     = "player";
   public static final String NODE_heights    = "heights";
   public static final String NODE_h          = "h";
   public static final String NODE_surfaces   = "surfaces";
   public static final String NODE_s          = "s";
   public static final String NODE_objects    = "objects";
   public static final String NODE_o          = "o";

   public static final String ATTR_version    = "version";
   public static final String ATTR_maxPlayers = "maxPlayers";
   public static final String ATTR_width      = "width";
   public static final String ATTR_height     = "height";
   public static final String ATTR_altFactor  = "altFactor";
   public static final String ATTR_waterLevel = "waterLevel";

   public static final String ATTR_v          = "v";
   public static final String ATTR_num        = "num";
   public static final String ATTR_posx       = "posx";
   public static final String ATTR_posy       = "posy";

   public GbmXmlSaxHandler()
   {
      super();
   }

   public void startElement(
         String uri, String localName, String qName, Attributes attributes)
         throws SAXException
   {
      if (NODE_gbm.equals(qName))
      {
         terrain = new GTerrain();
         terrain.setVersion(getIntegerValue(attributes, ATTR_version));
         terrain.setMaxPlayers(getIntegerValue(attributes, ATTR_maxPlayers));
         terrain.setWidth(getIntegerValue(attributes, ATTR_width));
         terrain.setHeight(getIntegerValue(attributes, ATTR_height));
         terrain.setAltFactor(getIntegerValue(attributes, ATTR_altFactor));
         terrain.setWaterLevel(getIntegerValue(attributes, ATTR_waterLevel));
      }
      else if (NODE_player.equals(qName))
      {
         currentGplayer = new GPlayer();
         terrain.addPlayer(currentGplayer);
         currentGplayer.setPlayerNumber(getIntegerValue(attributes, ATTR_num));
         currentGplayer.setTeamNumber(getIntegerValue(attributes, ATTR_num));
         currentGplayer.setStartPosX(getIntegerValue(attributes, ATTR_posx));
         currentGplayer.setStartPosX(getIntegerValue(attributes, ATTR_posy));
      }
      else if (NODE_heights.equals(qName))
      {
         terrain.setTerrainHeight(new float[terrain.getHeight()
               * terrain.getWidth()]);
         i = 0;
      }
      else if (NODE_h.equals(qName))
      {
         terrain.getTerrainHeight()[i] = getFloatValue(attributes, ATTR_v) / 10;
         i++;
      }
      else if (NODE_surfaces.equals(qName))
      {
         terrain.setSurfaces(new int[terrain.getHeight() * terrain.getWidth()]);
         i = 0;
      }
      else if (NODE_s.equals(qName))
      {
         terrain.getSurfaces()[i] = getIntegerValue(attributes, ATTR_v);
         i++;
      }
      else if (NODE_objects.equals(qName))
      {
         terrain.setObjects(new int[terrain.getHeight() * terrain.getWidth()]);
         i = 0;
      }
      else if (NODE_o.equals(qName))
      {
         terrain.getObjects()[i] = getIntegerValue(attributes, ATTR_v);
         i++;
      }
      else
      {
         throw new SAXException("Cannot handle node named '" + qName + "'");
      }

   }

   public void endElement(String uri, String localName, String qName)
         throws SAXException
   {

   }

   public InputSource resolveEntity(String publicId, String systemId)
         throws SAXException
   {
      InputSource inputSource = new InputSource();
      systemId = "g3d.dtd";
      inputSource.setSystemId(systemId);
      // inputSource.setPublicId("messungsdatei");
      return inputSource;
   }

   public GTerrain getTerrain()
   {
      return terrain;
   }

}
