package de.titusgames.jglest.xml.parser;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.titusgames.jglest.tileset.PathChance;
import de.titusgames.jglest.tileset.Tileset;
import de.titusgames.jglest.tileset.TilesetFog;
import de.titusgames.jglest.tileset.TilesetObject;
import de.titusgames.jglest.tileset.TilesetSound;
import de.titusgames.jglest.tileset.TilesetSurface;
import de.titusgames.jglest.tileset.TilesetWeather;

public class TilesetXmlSaxHandler extends XmlSaxHandlerBase
{
   URL baseUrl;
   Tileset                    tileset;
   TilesetSurface             tilesetSurface;
   TilesetObject              tilesetObject;
   TilesetSound               tilesetSound;
   PathChance                 pathChance;

   String                     currentMasterNode    = "";
   int                        i                    = 0;

   public static final String NODE_tileset         = "tileset";
   public static final String NODE_surfaces        = "surfaces";
   public static final String NODE_surface         = "surface";
   public static final String NODE_texture         = "texture";
   public static final String NODE_object          = "object";
   public static final String NODE_objects         = "objects";
   public static final String NODE_model           = "model";
   public static final String NODE_ambient_sounds  = "ambient-sounds";
   public static final String NODE_daySound        = "day-sound";
   public static final String NODE_nightSound      = "night-sound";
   public static final String NODE_rainSound       = "rain-sound";
   public static final String NODE_snowSound       = "snow-sound";
   public static final String NODE_dayStartSound   = "day-start-sound";
   public static final String NODE_nightStartSound = "night-start-sound";
   public static final String NODE_parameters      = "parameters";
   public static final String NODE_water           = "water";
   public static final String NODE_fog             = "fog";
   public static final String NODE_sunLight        = "sun-light";
   public static final String NODE_moonLight       = "moon-light";
   public static final String NODE_dayNight        = "day-night";
   public static final String NODE_weather         = "weather";

   public static final String ATTR_path            = "path";
   public static final String ATTR_prob            = "prob";
   public static final String ATTR_walkable        = "walkable";
   public static final String ATTR_enabled         = "enabled";
   public static final String ATTR_playAlways      = "play-always";
   public static final String ATTR_effects         = "effects";

   public static final String ATTR_mode            = "mode";
   public static final String ATTR_density         = "density";
   public static final String ATTR_colorRed        = "color-red";
   public static final String ATTR_colorGreen      = "color-green";
   public static final String ATTR_colorBlue       = "color-blue";
   public static final String ATTR_red             = "red";
   public static final String ATTR_green           = "green";
   public static final String ATTR_blue            = "blue";
   public static final String ATTR_value           = "value";

   public static final String ATTR_sun             = "sun";
   public static final String ATTR_rain            = "rain";
   public static final String ATTR_snow            = "snow";

   public TilesetXmlSaxHandler( URL baseUrl)
   {
      super();
      this.baseUrl = baseUrl;
   }

   public void startElement(
         String uri, String localName, String qName, Attributes attributes)
         throws SAXException
   {
      if (NODE_tileset.equals(qName))
      {
        tileset=new Tileset(baseUrl);
      }
      else if (NODE_surfaces.equals(qName))
      {
         // ignore;
         currentMasterNode = NODE_surface;
      }
      else if (NODE_surface.equals(qName))
      {
         tilesetSurface = new TilesetSurface();
         tileset.addSurfaces(tilesetSurface);
      }
      else if (NODE_texture.equals(qName)
            && NODE_surface.equals(currentMasterNode))
      {
         pathChance = new PathChance(attributes.getValue(ATTR_path),
               getFloatValue(attributes, ATTR_prob));
         tilesetSurface.addTextures(pathChance);
      }
      else if (NODE_objects.equals(qName))
      {
         // ignore;
      }
      else if (NODE_object.equals(qName))
      {
         tilesetObject = new TilesetObject();
         tileset.addObject(tilesetObject);
         tilesetObject.setWalkable(getBooleanValue(attributes, ATTR_walkable));
      }
      else if (NODE_model.equals(qName))
      {
         tilesetObject.addModelPath(attributes.getValue(ATTR_path));
      }
      else if (NODE_ambient_sounds.equals(qName))
      {
         // do nothing
      }
      else if ((NODE_daySound.equals(qName)) || (NODE_nightSound.equals(qName))
            || (NODE_rainSound.equals(qName)) || (NODE_snowSound.equals(qName))
            || (NODE_dayStartSound.equals(qName))
            || (NODE_nightStartSound.equals(qName)))
      {
         tilesetSound = new TilesetSound(qName);
         tileset.putSound(qName, tilesetSound);
         tilesetSound.setEnabled(getBooleanValue(attributes, ATTR_enabled));
         if (tilesetSound.isEnabled())
         {
            tilesetSound.setPath(attributes.getValue(ATTR_path));
            Boolean playAlways = getBooleanValue(
                  attributes, attributes.getValue(ATTR_playAlways));
            if (playAlways != null)
               tilesetSound.setPlayAlways(playAlways);
         }
      }
      else if (NODE_parameters.equals(qName))
      {
         // do nothing
      }
      else if (NODE_water.equals(qName))
      {
         tileset.getParameters().getWater().setEffects(
               getBooleanValue(attributes, ATTR_effects));
         currentMasterNode = NODE_water;
      }
      else if (NODE_texture.equals(qName)
            && NODE_water.equals(currentMasterNode))
      {
         tileset.getParameters().getWater().addTexturePath(
               attributes.getValue(ATTR_path));
      }
      else if (NODE_fog.equals(qName))
      {
         TilesetFog fog = new TilesetFog();
         tileset.getParameters().setFog(fog);
         fog.setEnabled(getBooleanValue(attributes, ATTR_enabled));
         if (fog.isEnabled())
         {
            fog.setColorRGB(getVector3dValue(
                  attributes, ATTR_colorRed, ATTR_colorGreen, ATTR_colorBlue));
         }
      }
      else if (NODE_sunLight.equals(qName))
      {
         tileset.getParameters().setSunlightRGB(
               getVector3dValue(attributes, ATTR_red, ATTR_green, ATTR_blue));
      }
      else if (NODE_moonLight.equals(qName))
      {
         tileset.getParameters().setMoonlightRGB(
               getVector3dValue(attributes, ATTR_red, ATTR_green, ATTR_blue));
      }
      else if (NODE_dayNight.equals(qName))
      {
         tileset.getParameters().setDaynight(
               getFloatValue(attributes, ATTR_value));
      }
      else if (NODE_weather.equals(qName))
      {
         TilesetWeather weather = new TilesetWeather(getFloatValue(
               attributes, ATTR_sun), getFloatValue(attributes, ATTR_rain),
               getFloatValue(attributes, ATTR_snow));

         tileset.getParameters().setWeather(weather);
      }
      else
      {
         throw new SAXException("Cannot handle node named '" + qName + "'");
      }

   }

   public InputSource resolveEntity(String publicId, String systemId)
         throws SAXException
   {
      InputSource inputSource = new InputSource();
      systemId = "g3d.dtd";
      inputSource.setSystemId(systemId);
      // inputSource.setPublicId("messungsdatei");
      return inputSource;
   }

   public Tileset getTileset()
   {
      return tileset;
   }

}
